@@ -1,4 +1,4 @@
-/* $Id: glibmm.h 345 2006-11-21 10:02:37Z murrayc $ */
+/* $Id: glibmm.h 417 2007-06-12 13:38:39Z murrayc $ */
 
 /* glibmm - a C++ wrapper for the GLib toolkit
  *
@@ -38,6 +38,7 @@
 #include <glibmm/interface.h>
 #include <glibmm/iochannel.h>
 #include <glibmm/init.h>
+#include <glibmm/keyfile.h>
 #include <glibmm/streamiochannel.h>
 #include <glibmm/listhandle.h>
 #include <glibmm/main.h>
@@ -53,6 +54,7 @@
 #include <glibmm/propertyproxy.h>
 #include <glibmm/quark.h>
 #include <glibmm/random.h>
+#include <glibmm/regex.h>
 #include <glibmm/refptr.h>
 #include <glibmm/shell.h>
 #include <glibmm/signalproxy_connectionnode.h>
