@@ -232,38 +232,45 @@ sub parse_on_cdata($$)
 
 
 # $strCommentBlock lookup_documentation($strFunctionName)
-sub lookup_documentation($)
+sub lookup_documentation($$)
 {
-  my ($functionName) = @_;
-   
+  my ($functionName, $deprecation_docs) = @_;
+
   my $objFunction = $DocsParser::hasharrayFunctions{$functionName};
   if(!$objFunction)
   {
     #print "DocsParser.pm: Warning: function not found: $functionName\n";
     return ""
   }
-  
+
   my $text = $$objFunction{description};
 
   if(length($text) eq 0)
   {
     print "DocsParser.pm: Warning: No C docs for function: \"$functionName\"\n";
   }
-  
-      
+
+
   DocsParser::convert_docs_to_cpp($objFunction, \$text);
+
+  #Add note about deprecation if we have specified that in our _WRAP_METHOD() call:
+  if($deprecation_docs ne "")
+  {
+    $text .= "\n\@deprecated $deprecation_docs";
+  }
+
   DocsParser::append_parameter_docs($objFunction, \$text);
   DocsParser::append_return_docs($objFunction, \$text);
 
-    
+
   # Escape the space after "i.e." or "e.g." in the brief description.
   $text =~ s/^([^.]*\b(?:i\.e\.|e\.g\.))\s/$1\\ /;
-      
+
   # Convert to Doxygen-style comment.
   $text =~ s/\n/\n${DocsParser::commentMiddleStart}/g;
   $text =  $DocsParser::commentStart . $text;
   $text .= "\n${DocsParser::commentEnd}\n";
-      
+
   return $text;
 }
 
@@ -364,6 +371,10 @@ sub convert_tags_to_doxygen($)
     s"Since: 2\.6"\@newin2p6"mg;
     s"Since: 2\.8"\@newin2p8"mg;
     s"Since: 2\.10"\@newin2p10"mg;
+    s"Since: 2\.12"\@newin2p12"mg;
+    s"Since: 2\.14"\@newin2p14"mg;
+    s"Since: 2\.16"\@newin2p16"mg;
+    s"Since: 2\.18"\@newin2p18"mg;
 
     s"\b-&gt;\b"->"g;
 
